/*
 * adcOpt.c
 *
 * Created: 8/7/2012 9:28:39 PM
 *  Author: Tim
 */ 

#include "adc.h"
#include "utility.h"




void adcInit()
{
	
	ADCA.CALL = ReadCalibrationByte( offsetof(NVM_PROD_SIGNATURES_t, ADCACAL0) );
	ADCA.CALH = ReadCalibrationByte( offsetof(NVM_PROD_SIGNATURES_t, ADCACAL1) );
	//delay of analog startup
	MCU.ANAINIT = MCU_STARTUPDLYA0_bm | MCU_STARTUPDLYA1_bm;
	
	//select the 2.5V external reference
	ADCA.REFCTRL	=	ADC_REFSEL_AREFB_gc		;
	
	//ADC prescaler selection
	ADCA.PRESCALER = ADC_PRESCALER_DIV128_gc;
	
	//setup the ADCA configuration
	ADCA.CTRLB		=
	ADC_IMPMODE_bm			|	//high impedance enable, since the board uses op-amps for the front end
	ADC_CURRLIMIT_NO_gc		|	//eliminate the current limit in order to max out the ADC speed
	ADC_CONMODE_bm			|	//sets up the conversions for signed mode
	ADC_FREERUN_bm			|	//sets up the adc to collect data autonomously
	ADC_RESOLUTION_12BIT_gc	;	//sets adc to have 12 bit resolution with right adjustment (LSB)
	
	//tell the ADC to speep all channels 0-3 automatically
	ADCA.EVCTRL = ADC_SWEEP_0123_gc;
	
	/**************Channel setups***************/
	
	
	//preliminary gain and input mode selection
	
	ADCA.CH0.CTRL	 = ADC_CH_GAIN_4X_gc | ADC_CH_INPUTMODE_DIFFWGAIN_gc;

	ADCA.CH1.CTRL	 = ADC_CH_GAIN_4X_gc | ADC_CH_INPUTMODE_DIFFWGAIN_gc;

	ADCA.CH2.CTRL	 = ADC_CH_GAIN_4X_gc | ADC_CH_INPUTMODE_DIFFWGAIN_gc;

	ADCA.CH3.CTRL	 = ADC_CH_GAIN_4X_gc | ADC_CH_INPUTMODE_DIFFWGAIN_gc;

	// mux positions
	
	ADCA.CH2.MUXCTRL =	ADC_CH_MUXPOS_PIN3_gc	|
	ADC_CH_MUXNEG_PIN7_gc	;
	
	ADCA.CH3.MUXCTRL =	ADC_CH_MUXPOS_PIN0_gc	|
	ADC_CH_MUXNEG_PIN4_gc	;
	
	ADCA.CH1.MUXCTRL =	ADC_CH_MUXPOS_PIN2_gc	|
	ADC_CH_MUXNEG_PIN6_gc	;
	
	ADCA.CH0.MUXCTRL =	ADC_CH_MUXPOS_PIN1_gc	|
	ADC_CH_MUXNEG_PIN5_gc	;
	
	ADCA.CH0.INTCTRL = ADC_CH_INTMODE_COMPLETE_gc | ADC_CH_INTLVL0_bm;
	ADCA.CH1.INTCTRL = ADC_CH_INTMODE_COMPLETE_gc | ADC_CH_INTLVL0_bm;
	ADCA.CH2.INTCTRL = ADC_CH_INTMODE_COMPLETE_gc | ADC_CH_INTLVL0_bm;
	ADCA.CH3.INTCTRL = ADC_CH_INTMODE_COMPLETE_gc | ADC_CH_INTLVL0_bm;
	
	ADCA.CH0.CTRL |= ADC_CH_START_bm;
	ADCA.CH1.CTRL |= ADC_CH_START_bm;
	ADCA.CH2.CTRL |= ADC_CH_START_bm;
	ADCA.CH3.CTRL |= ADC_CH_START_bm;
	
	ADCA.CTRLA |= ADC_ENABLE_bm;

	//set the appropriate interrupt levels at the global scope
	PMIC.CTRL |= PMIC_LOLVLEN_bm | PMIC_RREN_bm;
}

int16_t filterData(int16_t *data)
{
	int32_t sum = 0;
	uint8_t inc;
	
	for(inc=0;inc<16;inc++)
	{
		sum+=data[inc];
	}

	if (sum < 0)
	{
		sum = sum>>2;
		sum |= 0xC0000000;
	}
	else
	{
		sum = sum>>2;
	}
	return (int16_t) sum;
}

int16_t analogRead(uint8_t pinNum)
{
	if (pinNum > 3) return 0;
	else			return filterData(&adc_data[pinNum][0]);
}


ISR(ADCA_CH0_vect)
{
	adc_data[0][adcCnt[0].CNT++] = ADCA.CH0.RES;
}
ISR(ADCA_CH1_vect)
{
	adc_data[1][adcCnt[1].CNT++] = ADCA.CH1.RES;
}
ISR(ADCA_CH2_vect)
{
	adc_data[2][adcCnt[2].CNT++] = ADCA.CH2.RES;
}
ISR(ADCA_CH3_vect)
{
	adc_data[3][adcCnt[3].CNT++] = ADCA.CH3.RES;
}
